/*
 *  U6eFunctions.java
 *  
 *  Demonstrates the UD E-functions with the LabJack U6 with the LJUDJava
 *  JNI wrapper.
 *
 *  support@labjack.com
 *  July 21, 2008
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class U6eFunctions {

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception {
		int errorcode;
		double[] dblValue = { 0 };
		int[] value = { 0 };
		int[] handle = { 0 };
		int binary;
		int[] aEnableTimers = new int[2];
		int[] aEnableCounters = new int[2];
		int TCPinOffset, timerClockBaseIndex, timerClockDivisor;
		int[] aTimerModes = new int[2];
		double[] adblTimerValues = new double[2];
		int[] aReadTimers = new int[2];
		int[] aUpdateResetTimers = new int[2];
		int[] aReadCounters = new int[2];
		int[] aResetCounters = new int[2];
		double[] adblCounterValues = new double[2];
		double highTime, lowTime, dutyCycle;

		NumberFormat formatter = new DecimalFormat("0.000");

		// Read and display the UD and LJUDJava versions.
		System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
		System.out.println("LJUD JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");

		// Open the first found LabJack U6.
		errorcode = LJUD.OpenLabJack(LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "1", 1, handle);
		ErrorHandler(errorcode, 0, new Exception());

		// Take a measurement from AIN0.
		binary = 0;
		errorcode = LJUD.eAIN(handle[0], 0, 199, dblValue, -1, -1, -1, binary, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("AIN0 = " + dblValue[0]);

		// Set DAC0 to 3.0 volts.
		dblValue[0] = 3.0;
		binary = 0;
		errorcode = LJUD.eDAC(handle[0], 0, dblValue[0], binary, 0, 0);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("DAC0 set to " + dblValue[0] + " volts");

		// Read state of FIO1.
		errorcode = LJUD.eDI(handle[0], 1, value);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("FIO1 = " + value[0]);

		// Set the state of FIO2.
		value[0] = 1;
		errorcode = LJUD.eDO(handle[0], 2, value[0]);
		ErrorHandler(errorcode, 0, new Exception());
		System.out.println("FIO2 set to = " + value[0] + "\n");
	}

	public U6eFunctions() {
	}

	public static void main(String[] args) {
		try {
			new U6eFunctions().runExample();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}