//---------------------------------------------------------------------------
//
//  TwoUnits.java
//  
//  Simple example demonstrates communication with 2 U6s.
//
//  support@labjack.com
//  June 26, 2009
//----------------------------------------------------------------------
//

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class TwoUnits {

	public static void main(String[] args) 
	{
		try 
		{
			new TwoUnits().runExample();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
	}

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static {
		try {
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} catch (UnsatisfiedLinkError e) {
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) {
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) {
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception
	{
		int intErrorcode;
		long lngIteration;
		long lngInnerIteration;
		double dblDriverVersion;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		double Value12=9999,Value22=9999,Value32=9999;
		double Value13=9999,Value23=9999,Value33=9999;
		int[] intHandle2={0},intHandle3={0};
		String input = new String();
		boolean finished = false;

		// Scanner to read input
		Scanner s = new Scanner(System.in);

		// Variables to satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		//Read and display the UD version.
		dblDriverVersion = LJUD.GetDriverVersion();
		System.out.println("UD Driver Version = " + dblDriverVersion + "\n");

		//Open the U6 with local ID 2.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "2", 0, intHandle2);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Open the U6 with local ID 3.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtU6, LJUD.LJ_ctUSB, "3", 0, intHandle3);
		ErrorHandler(intErrorcode, 0, new Exception());

		//The following commands will use the add-go-get method to group
		//multiple requests into a single low-level function.

		//Request a single-ended reading from AIN1.
		intErrorcode = LJUD.AddRequest (intHandle2[0], LJUD.LJ_ioGET_AIN, 1, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle3[0], LJUD.LJ_ioGET_AIN, 1, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Request a single-ended reading from AIN2.
		intErrorcode = LJUD.AddRequest (intHandle2[0], LJUD.LJ_ioGET_AIN, 2, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle3[0], LJUD.LJ_ioGET_AIN, 2, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Request a reading from AIN3 using the Special 0-3.6 range.
		intErrorcode = LJUD.AddRequest (intHandle2[0], LJUD.LJ_ioGET_AIN_DIFF, 3, 0, 199, 0);
		ErrorHandler(intErrorcode, 0, new Exception());
		intErrorcode = LJUD.AddRequest (intHandle3[0], LJUD.LJ_ioGET_AIN_DIFF, 3, 0, 199, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		lngIteration = 0;

		while (!finished)
		{

			//Execute all requests on all open LabJacks.
			intErrorcode = LJUD.Go ();
			ErrorHandler(intErrorcode, lngIteration, new Exception());

			//Get all the results for unit 2.  The input measurement results are stored.
			intErrorcode = LJUD.GetFirstResult(intHandle2[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			ErrorHandler(intErrorcode, lngIteration, new Exception());

			lngInnerIteration=0;	//Used by the error handling function.

			while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
			{
				if (intIOType[0] == LJUD.LJ_ioGET_AIN)
				{
					switch(intChannel[0])
					{
						case 1:
							Value12=dblValue[0];
							break;
						case 2:
							Value22=dblValue[0];
							break;
					}
				}

				else if (intIOType[0] == LJUD.LJ_ioGET_AIN_DIFF)
					Value32=dblValue[0];

				intErrorcode = LJUD.GetNextResult(intHandle2[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
				if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
				{
					ErrorHandler(intErrorcode, lngInnerIteration, new Exception());
				}

				lngInnerIteration++;

			}

			//Get all the results for unit 3.  The input measurement results are stored.
			intErrorcode = LJUD.GetFirstResult(intHandle3[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			ErrorHandler(intErrorcode, lngIteration, new Exception());

			lngInnerIteration=0;	//Used by the error handling function.

			while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
			{
				if (intIOType[0] == LJUD.LJ_ioGET_AIN)
				{
					switch(intChannel[0])
					{
						case 1:
							Value13=dblValue[0];
							break;
						case 2:
							Value23=dblValue[0];
							break;
					}
				}

				else if (intIOType[0] == LJUD.LJ_ioGET_AIN_DIFF)
					Value33=dblValue[0];

				intErrorcode = LJUD.GetNextResult(intHandle3[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
				if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
				{
					ErrorHandler(intErrorcode, lngInnerIteration, new Exception());
				}

				lngInnerIteration++;

			}

			System.out.println("AIN1 (Unit 2) = " + Value12);
			System.out.println("AIN1 (Unit 3) = " + Value13);
			System.out.println("AIN2 (Unit 2) = " + Value22);
			System.out.println("AIN2 (Unit 3) = " + Value23);
			System.out.println("AIN3 (Unit 2) = " + Value32);
			System.out.println("AIN3 (Unit 3) = " + Value33);
	
			System.out.println("\nPress Enter to go again or (q) to quit\n");
			input = s.nextLine();
			finished = input.equals("q");
		}
	}
}