/*
 *  Simple.java
 *
 *  Basic command/response UE9 example using the UD driver with the 
 *  LJUDJava JNI wrapper.
 *
 *  support@labjack.com
 *  March 14, 2006
 */

import java.io.*;
import java.text.*;
import com.labjack.labjackud.LJUD;     //importing the LJUD class

public class Simple {
    
    //Loading the LJUDJava dll so that we can use the LJUDJava JNI.  This
    //example assumes that the dll is in the "../../../LJUDJava/" folder.
    static {
        try {
            System.loadLibrary("../../../LJUDJava/LJUDJava");
        } catch (UnsatisfiedLinkError e) {
            System.err.println("Native code library failed to load.\n" + e);
            System.exit(1);
        }
    }
    
    //This is our simple error handling function that is called after every UD
    //function call.  This function displays the errorcode and string description
    //of the error.  It also has an iteration input is useful when processing
    //results in a loop (getfirst/getnext).  A stack trace is also displayed so
    //that you can find the line of code where the error occurred.
    public static void ErrorHandler(int Errorcode, int Iteration, Exception excep) {
        byte []err = new byte[255];
        
        if (Errorcode != LJUD.LJE_NOERROR) {
            LJUD.ErrorToString(Errorcode, err);
            System.out.println("Error number = " + Errorcode);
            System.out.println("Error string = " + new String(err));
            System.out.println("Iteration = " + Iteration);
            System.out.println("Stack Trace : ");
            excep.printStackTrace();
            if(Errorcode > LJUD.LJE_MIN_GROUP_ERROR) {
                //Quit if this is a group error.
                System.exit(1);
            }
        }
    }
    
    public void runExample() throws Exception {
        
        int Errorcode = 0;
        int []IOType = {0};
        int []Channel = {0};
        double []Value = {0};
        double Value2 = 0, Value3 = 0;
        double ValueDIBit = 0,ValueDIPort = 0,ValueCounter = 0;
        int []Handle = {0};
        int GetNextIteration;
        char ch;
        double []dZero = {0};
        int []iZero = {0};
        NumberFormat formatter = new DecimalFormat("0.000");
        BufferedReader keyboard = new
                BufferedReader(new InputStreamReader(System.in));
        
        
	//Read and display the UD and LJUDJava versions.
	System.out.println("UD Driver Version = " + formatter.format(LJUD.GetDriverVersion()));
        System.out.println("LJUDJava JNI Version = " + formatter.format(LJUD.LJUDJAVA_VERSION) + "\n");
        
        //Open the first found LabJack UE9.
        Errorcode = LJUD.OpenLabJack(LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, Handle);
        //Errorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, Handle);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //First some configuration commands.  These will be done with the ePut
        //function which combines the add/go/get into a single call.
        
        //Configure for 16-bit analog input measurements.
        Errorcode = LJUD.ePut(Handle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chAIN_RESOLUTION, 16, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Configure the analog input range on channels 2 and 3 for bipolar gain=1.
        Errorcode = LJUD.ePut(Handle[0],  LJUD.LJ_ioPUT_AIN_RANGE, 2, LJUD.LJ_rgBIP5V, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        Errorcode = LJUD.ePut(Handle[0],  LJUD.LJ_ioPUT_AIN_RANGE, 3, LJUD.LJ_rgBIP5V, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Enable Counter0 which will appear on FIO0 (assuming no other
        //program has enabled any timers or Counter1).
        Errorcode = LJUD.ePut(Handle[0],  LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        
        //Now we add requests to write and read I/O.  These requests
        //will be processed repeatedly by go/get statements in every
        //iteration of the while loop below.
        
        //Request AIN2 and AIN3.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_AIN, 2, 0, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_AIN, 3, 0, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Set DAC0 to 2.5 volts.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_DAC, 0, 2.5, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Read digital input FIO1.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_DIGITAL_BIT, 1, 0, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Set digital output FIO2 to output-high.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioPUT_DIGITAL_BIT, 2, 1, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Read digital inputs FIO3 through FIO7.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_DIGITAL_PORT, 3, 0, 5, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        //Request the value of Counter0.
        Errorcode = LJUD.AddRequest(Handle[0], LJUD.LJ_ioGET_COUNTER, 0, 0, 0, 0);
        ErrorHandler(Errorcode, 0, new Exception());
        
        
        while (true) {
            //Execute the requests.
            Errorcode = LJUD.GoOne(Handle[0]);
            ErrorHandler(Errorcode,  0, new Exception());
            
            
            //Get all the results.  The input measurement results are stored.  All other
            //results are for configuration or output requests so we are just checking
            //whether there was an error.
            Errorcode = LJUD.GetFirstResult(Handle[0], IOType, Channel, Value, iZero, dZero);
            ErrorHandler(Errorcode,  0, new Exception());
            
            GetNextIteration=0;	//Used by the error handling function.
            
            while(Errorcode < LJUD.LJE_MIN_GROUP_ERROR) {
                if(IOType[0] == LJUD.LJ_ioGET_AIN) {
                    if(Channel[0] == 2)
                        Value2=Value[0];
                    if(Channel[0] == 3)
                        Value3=Value[0];
                }
                if(IOType[0] == LJUD.LJ_ioGET_DIGITAL_BIT)
                    ValueDIBit=Value[0];
                
                if(IOType[0] == LJUD.LJ_ioGET_DIGITAL_PORT)
                    ValueDIPort=Value[0];
                
                if(IOType[0] ==  LJUD.LJ_ioGET_COUNTER)
                    ValueCounter=Value[0];
                
                Errorcode = LJUD.GetNextResult(Handle[0], IOType, Channel, Value, iZero, dZero);
                if(Errorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE) {
                    ErrorHandler(Errorcode, GetNextIteration, new Exception());
                }
                
                GetNextIteration++;
            }
            
            System.out.println("AIN2 = " + formatter.format(Value2));
            System.out.println("AIN3 = " + formatter.format(Value3));
            System.out.println("FIO1 = " + ValueDIBit);
            System.out.println("FIO3-FIO7 = " + ValueDIPort);  //Will read 31 if all 5 lines are pulled-high as normal.
            System.out.println("Counter0 (FIO0) = " + ValueCounter);
            
            System.out.println("\nPress Enter to go again or (q) and Enter to quit\n");
            ch = (char)keyboard.read();
            if (ch == 'q') return;
  
            System.out.println("\nDone");            
        }
    }
    
    public Simple() {}
    
    public static void main(String[] args) {
        try {
            new Simple().runExample();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

