//---------------------------------------------------------------------------
//
//  TimerCounter.java
//  
//  Basic UE9 example does a PWM output and a counter input, using AddGoGet method.
//
//  support@labjack.com
//  June 29, 2009
//----------------------------------------------------------------------

import java.io.*;
import java.text.*;
import java.util.Scanner;
import com.labjack.labjackud.LJUD; // importing the LJUD class

public class TimerCounter 
{

	public static void main(String[] args) 
	{
		try 
		{
			new TimerCounter().runExample();
		} 
		catch (Exception ex) 
		{
			ex.printStackTrace();
		}
	}

	// Loading the LJUDJava dll so that we can use the LJUDJava interface. This
	// example assumes that the dll is in the "../../../LJUDJava/" folder.
	static 
	{
		try 
		{
			System.loadLibrary("../../../LJUDJava/LJUDJava");
		} 
		catch (UnsatisfiedLinkError e) 
		{
			System.err.println("Native code library failed to load.\n" + e);
			System.exit(1);
		}
	}

	// This is our simple error handling function that is called after every UD
	// function call. This function displays the errorcode and string
	// description of the error. It also has an iteration input is useful when 
	// processing results in a loop (getfirst/getnext). A stack trace is also
	// displayed so that you can find the line of code where the error occurred.
	public static void ErrorHandler(int Errorcode, long Iteration, Exception excep) 
	{
		byte[] err = new byte[255];

		if (Errorcode != LJUD.LJE_NOERROR) 
		{
			LJUD.ErrorToString(Errorcode, err);
			System.out.println("Error number = " + Errorcode);
			System.out.println("Error string = " + new String(err));
			System.out.println("Iteration = " + Iteration);
			System.out.println("Stack Trace : ");
			excep.printStackTrace();
			if (Errorcode > LJUD.LJE_MIN_GROUP_ERROR) 
			{
				// Quit if this is a group error.
				System.exit(1);
			}
		}
	}

	public void runExample() throws Exception
	{
		int intErrorcode;
		long lngGetNextIteration;
		int[] intIOType={0}, intChannel={0};
		double[] dblValue={0};
		int[] intHandle={0};
		long resetCounter=1;
		double count=-1;
		double highTime,lowTime;
		double period16 = 0,period32 = 0;

		// Variables to satisfy certain method signatures
		int[] dummyInt = {0};
		double[] dummyDouble = {0};

		//Open the LabJack UE9.
		intErrorcode = LJUD.OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctUSB, "1", 1, intHandle);
		//errorcode = OpenLabJack (LJUD.LJ_dtUE9, LJUD.LJ_ctETHERNET, "192.168.1.209", 0, Handle);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Disable all timers and counters to put everything in a known initial state.
		//Disable the timer and counter, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 1, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);


		//First we will output a square wave and count the number of pulses for about 1 second.
		//Connect a jumper on the UE9 from FIO0 (PWM output) to
		//FIO1 (Counter0 input).

		//Use the fixed 750kHz timer clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc750KHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 3 so the actual timer clock is 250kHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 1 timer.  It will use FIO0.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable Counter0.  It will use FIO1 since 1 timer is enabled.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 1, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;
		}


		//Wait 1 second.
		Thread.sleep(1000);

		//Request a read from the counter.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_COUNTER, 0, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//This should read roughly 977 counts.
		System.out.println("Counter = " + dblValue[0] + "\n");

		//Disable the timer and counter, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);




		//Output a square wave and measure the period.
		//Connect a jumper on the UE9 from FIO0 (PWM8 output) to
		//FIO1 (RISINGEDGES32 input) and FIO2 (RISINGEDGES16).

		//Use the fixed 750kHz timer clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc750KHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 3 so the actual timer clock is 250kHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 3 timers.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 50%.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 32768, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 as 32-bit period measurement.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmRISINGEDGES32, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer2 as 16-bit period measurement.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 2, LJUD.LJ_tmRISINGEDGES16, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;
		}


		//Wait 1 second.
		Thread.sleep(1000);

		//Now read the period measurements from the 2 timers.  We
		//will use the Add/Go/Get method so that both
		//reads are done in a single low-level call.

		//Request a read from Timer1
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_TIMER, 1, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Request a read from Timer2
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioGET_TIMER, 2, 0, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Get the results of the two read requests.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			if (intIOType[0] == LJUD.LJ_ioGET_TIMER)
				switch(intChannel[0])
				{
					case 1:
						period32=dblValue[0];
						break;
					case 2:
						period16=dblValue[0];
						break;
				}

			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;

		}


		//Both period measurements should read about 256.  The timer
		//clock was set to 250 kHz, so each tick equals 4 microseconds, so
		//256 ticks means a period of 1024 microseconds which is a frequency
		//of 977 Hz.
		System.out.println("Period32 = " + period32);
		System.out.println("Period16 = " + period16 + "\n");

		//Disable the timer and counter, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_COUNTER_ENABLE, 0, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);




		//Now we will output a 25% duty-cycle PWM output on Timer0 (FIO0) and measure
		//the duty cycle on Timer1 FIO1.  Requires Control firmware V1.21 or higher.

		//Use the fixed 750kHz timer clock source.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_BASE, LJUD.LJ_tc750KHZ, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the divisor to 3 so the actual timer clock is 250kHz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chTIMER_CLOCK_DIVISOR, 3, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Enable 2 timers.  They will use FIO0 and FIO1.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 2, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer0 as 8-bit PWM.  Frequency will be 250k/256 = 977 Hz.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 0, LJUD.LJ_tmPWM8, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Set the PWM duty cycle to 25%.  The passed value is the low time.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_VALUE, 0, 49152, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//Configure Timer1 as duty cycle measurement.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_TIMER_MODE, 1, LJUD.LJ_tmDUTYCYCLE, 0, 0);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Execute the requests on a single LabJack.  The driver will use a
		//single low-level TimerCounter command to handle all the requests above.
		intErrorcode = LJUD.GoOne(intHandle[0]);
		ErrorHandler(intErrorcode, 0, new Exception());


		//Get all the results just to check for errors.
		intErrorcode = LJUD.GetFirstResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
		lngGetNextIteration=0;	//Used by the error handling function.
		while(intErrorcode < LJUD.LJE_MIN_GROUP_ERROR)
		{
			intErrorcode = LJUD.GetNextResult(intHandle[0], intIOType, intChannel, dblValue, dummyInt, dummyDouble);
			if(intErrorcode != LJUD.LJE_NO_MORE_DATA_AVAILABLE)
			{
				ErrorHandler(intErrorcode, lngGetNextIteration, new Exception());
			}

			lngGetNextIteration++;
		}


		//Wait a little so we are sure a duty cycle measurement has occured.
		Thread.sleep(100);

		//Request a read from Timer1.
		intErrorcode = LJUD.eGet(intHandle[0], LJUD.LJ_ioGET_TIMER, 1, dblValue, 0);
		ErrorHandler(intErrorcode, 0, new Exception());

		//High time is LSW
		highTime = (double)(dblValue[0] % (65536));
		//Low time is MSW
		lowTime = (double)(dblValue[0] / (65536));

		System.out.println("High clicks = " + highTime);
		System.out.println("Low clicks = " + lowTime);
		System.out.println("Duty cycle = " + (100*highTime/(highTime+lowTime)));


		//Disable the timers, and the FIO lines will return to digital I/O.
		intErrorcode = LJUD.AddRequest (intHandle[0], LJUD.LJ_ioPUT_CONFIG, LJUD.LJ_chNUMBER_TIMERS_ENABLED, 0, 0, 0);
		intErrorcode = LJUD.GoOne(intHandle[0]);


		//The PWM output sets FIO0 to output, so we do a read here to set
		//FIO0 to input.
		intErrorcode = LJUD.eGet (intHandle[0], LJUD.LJ_ioGET_DIGITAL_BIT, 0, dblValue, dummyDouble);
		ErrorHandler(intErrorcode, 0, new Exception());
	}
}